package de.ozgcloud.processor.vorgang;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.stream.Stream;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.itvsh.ozg.pluto.command.VorgangCreatedEvent;
import de.ozgcloud.apilib.vorgang.OzgCloudVorgangTestFactory;
import de.ozgcloud.processor.processor.ProcessorResult;
import de.ozgcloud.processor.processor.ProcessorResultTestFactory;
import de.ozgcloud.processor.processor.ProcessorService;
import de.ozgcloud.processor.result.ProcessorTechnicalException;
import de.ozgcloud.processor.result.ResultService;
import reactor.core.publisher.Mono;

class VorgangEventListenerTest {

	private static final VorgangCreatedEvent EVENT = new VorgangCreatedEvent(OzgCloudVorgangTestFactory.ID.toString());

	@Spy
	@InjectMocks
	private VorgangEventListener vorgangEventListener;

	@Mock
	private VorgangService vorgangService;
	@Mock
	private ProcessorService processorService;
	@Mock
	private ResultService resultService;
	@Mock
	private ProcessorTechnicalException exception;

	@Captor
	private ArgumentCaptor<ProcessorResult> resultCaptor;

	private Vorgang vorgang = VorgangTestFactory.create();

	@BeforeEach
	void init() {
		when(vorgangService.getVorgang(any())).thenReturn(vorgang);
		
		var result = ProcessorResultTestFactory.createBuilder().vorgangId(null).build();
		when(processorService.processVorgang(any())).thenReturn(Stream.of(Mono.just(result)));
	}

	@Test
	void shouldGetVorgang() {
		var expectedVorgangId = VorgangId.from(OzgCloudVorgangTestFactory.ID.toString());

		vorgangEventListener.triggerNewVorgangProcessors(EVENT);

		verify(vorgangService).getVorgang(expectedVorgangId);
	}

	@Test
	void shouldCallProcessVorgang() {
		vorgangEventListener.triggerNewVorgangProcessors(EVENT);

		verify(processorService).processVorgang(vorgang);
	}

	@Test
	void shouldSetVorgangId() {
		vorgangEventListener.triggerNewVorgangProcessors(EVENT);

		verify(resultService).processResult(resultCaptor.capture());
		assertThat(resultCaptor.getValue().getVorgangId()).isEqualTo(VorgangTestFactory.ID);
	}

	@Test
	void shouldCallConsumeException() {
		var vorgang = VorgangTestFactory.create();
		when(vorgangService.getVorgang(any())).thenReturn(vorgang);
		when(processorService.processVorgang(any())).thenReturn(Stream.of(Mono.error(exception)));

		vorgangEventListener.triggerNewVorgangProcessors(EVENT);

		verify(resultService).processError(exception, VorgangTestFactory.ID.toString());
	}

	@Test
	void shouldCallConsumeUnexpectedExceptionType() {
		var vorgang = VorgangTestFactory.create();
		when(vorgangService.getVorgang(any())).thenReturn(vorgang);
		when(processorService.processVorgang(any())).thenReturn(Stream.of(Mono.error(new Exception())));

		vorgangEventListener.triggerNewVorgangProcessors(EVENT);

		verify(resultService).processError(any(Exception.class), eq(VorgangTestFactory.ID.toString()));
	}
}